//////////////////////////////////////////////////
// File: TworkerThread.cpp
// Author: Kelvin Sung
// Date: April 2003
// 
// This example shows how to use CMutex and CEvent to support
// synchronization between the main thread and each worker thread
// to do work together.
//
//
//
#include "TworkerThread.h"

#include <iostream.h>
#include <iomanip.h>
			// for cout/cin/cerr, and i/o formatting

#include <fstream.h>
			// for ostream
			
#include <process.h>
			// for _beginthread and _endthread

//
// #inlcude <afxmt.h>
//			// for MFC synchronization objects: CMutex/CEvent
//	

//
// To Compile: for Both Debug and Release:
//    Must include compiler optins: /MT /D "_X86_"
//
// To Link: 
//          For Debug Must include:   LIBCMTD.lib and MSVCRTD.lib
//			For Release Must include: LibCMT.LIB and MSVCRT.LIB
//
//    NAFXCWD.lib (NAFXCW.LIB for Release) must appear _BEFORE_ LIBCMTD.LIB)
//
//
//

static void WorkerThread(void *parm)
	//
	// Check via TsyncObject, do when when told
	//
{
	TworkerThread *td = (TworkerThread *) parm;
	ofstream &os = td->OutStream();
	TsyncObject &sync = TworkerThread::GetSyncObject();


	while (!sync.ShouldQuit()) {
		#ifdef _DEBUG_DETAIL
				cerr << "Thread: [" << td->ThreadName() << "]: going to check for work" << endl;
		#endif
		if ( sync.CheckNextCommand(td->ThreadID()) ) {

			#ifdef _DEBUG_DETAIL
				cerr << "Thread: [" << td->ThreadName() << "]: Got Work!" << endl;
			#endif

			td->GotNewJob();

			os << "	Working on job (" << td->NumJob() <<")" << endl;
			os.flush();

			td->ThreadWorkTime();
			
			os << "	Done    on job (" << td->NumJob() << ")" << endl;
		} 
	}
	

	os << "Total Jobs Done: " << td->NumJob() << endl;

	_endthread();

}

///////////////////////////////////////////////////////////////////////////////////////
//
// TworkerThread methods
//
/////////////////////////////////////////////////////////////////////////////////////

		TsyncObject TworkerThread::sfSyncObj;
			// definition of class object


void
TworkerThread::StartThread()
{
	_beginthread(WorkerThread, 0, (void *) this );
}

